#pragma once

#include "componentbasic.h"
#include <messagebasic.h>
#include <QJsonObject>
#include <QJSEngine>

class MessageLoader;
class CompareDelegate;
class ConvertDelegate;
class ExecuteDelegate;
class LogicalNode;

/// <summary>
/// ͨ
/// </summary>
struct COMPONENTBASIC_EXPORT GeneralData : public TopicData {
	QJsonObject _element_bind;

	QString topicString() const override;
	std::shared_ptr<Serializable> newDefault() const override;

	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
};
// ==================================================================================

/// <summary>
/// Ϊʵڵ
/// </summary>
class COMPONENTBASIC_EXPORT MapKernal :
	public QObject, virtual public Serializable,
	public std::enable_shared_from_this<MapKernal> {
	Q_OBJECT
private:
	/// <summary>
	/// 
	/// </summary>
	std::shared_ptr<MessageLoader> _bind_loader;

	/// <summary>
	/// {Topicdefault-value}
	/// </summary>
	QHash<QString, std::shared_ptr<TopicData>> _variable_types;

	/// <summary>
	/// Ƚ
	/// </summary>
	QHash<QString, std::shared_ptr<CompareDelegate>> _compare_types;
	/// <summary>
	/// ת
	/// </summary>
	QHash<QString, std::shared_ptr<ConvertDelegate>> _convert_types;
	/// <summary>
	/// ִ
	/// </summary>
	QHash<QString, std::shared_ptr<ExecuteDelegate>> _execute_types;

	/// <summary>
	/// пõĽڵ
	/// </summary>
	QHash<QString, std::shared_ptr<LogicalNode>> _logicalnode_types;

signals:
	/// <summary>
	/// Ƚб
	/// </summary>
	void compareTypeListChanged();
	/// <summary>
	/// תб
	/// </summary>
	void convertTypeListChanged();
	/// <summary>
	/// ִнڵб
	/// </summary>
	void executeTypeListChanged();


public:
	MapKernal(std::shared_ptr<MessageLoader> ins);
	virtual ~MapKernal() = default;

	/// <summary>
	/// ȡMessageLoader
	/// </summary>
	/// <returns></returns>
	std::shared_ptr<MessageLoader> messageLoader() const;

	void initial();

	// ͹ ======================================================
	/// <summary>
	/// ע
	/// </summary>
	/// <param name="init"></param>
	virtual void registData(std::shared_ptr<TopicData> init);
	/// <summary>
	/// ȡп
	/// </summary>
	/// <returns></returns>
	virtual QList<QString> dataTypes() const;
	/// <summary>
	/// ȡָtopic
	/// </summary>
	/// <param name="topic"></param>
	/// <returns></returns>
	virtual std::shared_ptr<TopicData> getData(const QString& topic);




	// ߼͹ ====================================================
	/// <summary>
	/// ע߼ڵ
	/// </summary>
	/// <param name="ins"></param>
	virtual void registNode(std::shared_ptr<LogicalNode> ins);
	/// <summary>
	/// ִ
	/// </summary>
	/// <returns></returns>
	virtual QList<QString> nodeTypes() const;
	/// <summary>
	/// ȡָ͵ִڵ
	/// </summary>
	/// <param name="name"></param>
	/// <returns></returns>
	virtual std::shared_ptr<LogicalNode> getNode(const QString& name);




	// Ƚ͹ ======================================================
	/// <summary>
	/// עȽ
	/// </summary>
	/// <param name="ins"></param>
	virtual void registCompare(std::shared_ptr<CompareDelegate> ins);
	/// <summary>
	/// ȡȽ
	/// </summary>
	/// <returns></returns>
	virtual QList<QString> compareTypes() const;
	/// <summary>
	/// ȡȽʵ
	/// </summary>
	/// <param name="name"></param>
	/// <returns></returns>
	virtual std::shared_ptr<CompareDelegate> getCompare(const QString& name);




	// ת͹ ======================================================
	/// <summary>
	/// עת
	/// </summary>
	/// <param name="ins"></param>
	virtual void registConvert(std::shared_ptr<ConvertDelegate> ins);
	/// <summary>
	/// ת
	/// </summary>
	/// <returns></returns>
	virtual QList<QString> convertTypes() const;
	/// <summary>
	/// ȡָ͵ת
	/// </summary>
	/// <param name="name"></param>
	/// <returns></returns>
	virtual std::shared_ptr<ConvertDelegate> getConvert(const QString& name);



	// ִнڵ͹ ====================================================
	/// <summary>
	/// עչִ
	/// </summary>
	/// <param name="ins"></param>
	virtual void registExecute(std::shared_ptr<ExecuteDelegate> ins);
	/// <summary>
	/// ִ
	/// </summary>
	/// <returns></returns>
	virtual QList<QString> executeTypes() const;
	/// <summary>
	/// ȡָ͵ִڵ
	/// </summary>
	/// <param name="name"></param>
	/// <returns></returns>
	virtual std::shared_ptr<ExecuteDelegate> getExecute(const QString& name);



	// ͨ Serializable ̳
	std::shared_ptr<Serializable> newDefault() const override;
	void recoveryFrom(const QJsonObject& obj) override {}
	void saveTo(QJsonObject& obj) const override {}
};

/// <summary>
/// ȽϹϵʵ
/// </summary>
class COMPONENTBASIC_EXPORT CompareDelegate {
private:
	QJSEngine& _script_engine;
	QString _function_name;

public:
	CompareDelegate(QJSEngine& bind_engine, const QString& func);

	QString name() const;
	bool compare(std::shared_ptr<TopicData> vleft, std::shared_ptr<TopicData> vright);
};

/// <summary>
/// תʵ
/// </summary>
class COMPONENTBASIC_EXPORT ConvertDelegate {
public:
	typedef QString DATA_TOPIC_STRING;

	/// <summary>
	/// ת
	/// </summary>
	/// <param name="host">͹</param>
	/// <param name="bind_engine"></param>
	/// <param name="func"></param>
	ConvertDelegate(std::shared_ptr<MapKernal> host, QJSEngine& bind_engine, const QString& func);
	/// <summary>
	/// תƣJS
	/// </summary>
	/// <returns></returns>
	QString name() const;

	/// <summary>
	/// Ͱ󶨱
	/// </summary>
	/// <returns>list{pair{,}}</returns>
	QList<std::pair<DATA_TOPIC_STRING, QString>> inputTable() const;
	/// <summary>
	/// ͱ
	/// </summary>
	/// <param name="table">list{pair{,}}</param>
	void inputReset(const QList<std::pair<DATA_TOPIC_STRING, QString>>& table);
	/// <summary>
	/// 
	/// </summary>
	/// <returns>pair{,}</returns>
	std::pair<DATA_TOPIC_STRING, QString> outputVariable() const;
	/// <summary>
	/// 
	/// </summary>
	/// <param name="appoint"></param>
	void outputReset(std::pair<DATA_TOPIC_STRING, QString> appoint);

	/// <summary>
	/// ִת
	/// </summary>
	/// <param name="input_variables">б</param>
	/// <returns></returns>
	std::shared_ptr<TopicData> convert(const QList<std::shared_ptr<TopicData>>& input_variables);

private:
	std::shared_ptr<MapKernal> _host_bind;

	QJSEngine& _script_engine;
	QString _function_name;

	QList<std::pair<DATA_TOPIC_STRING, QString>> _input_table;
	std::pair<DATA_TOPIC_STRING, QString> _output_appoint;
};

/// <summary>
/// ִн
/// </summary>
enum class LogicalResult {
	UNDEFINED,	// δ
	SUCCESS,	// ɹ
	FAILURE		// ʧ
};
/// <summary>
/// ִд
/// </summary>
class COMPONENTBASIC_EXPORT ExecuteDelegate : public Serializable {
public:
	/// <summary>
	/// ȡڵ
	/// </summary>
	/// <returns></returns>
	virtual QString typeName() const = 0;
	/// <summary>
	/// ڲʶݽӿ
	/// </summary>
	/// <returns>map{name, type-ins}</returns>
	virtual QHash<QString, std::shared_ptr<TopicData>> inputList() const = 0;
	/// <summary>
	/// ڲʶݽӿ
	/// </summary>
	/// <returns>map{name, type-ins}</returns>
	virtual QHash<QString, std::shared_ptr<TopicData>> outputList() const = 0;


	/// <summary>
	/// ڲ״̬
	/// </summary>
	virtual void reset() = 0;
	/// <summary>
	/// ִ߼ڵ
	/// </summary>
	/// <param name="core"></param>
	/// <returns></returns>
	virtual LogicalResult execute() = 0;
};


/// <summary>
/// ߼ڵ
/// </summary>
enum class NodeKind {
	MAPNODE,		// ߼ڵ
	SEQUENCENODE,	// ˳ڵ
	SELECTORNODE,	// ѡڵ
	PARALLELNODE,	// нڵ
	COMPARENODE,	// ȽϽڵ
	ACTIONNODE,		// ڵ
};
class BehaviorMapNode;
/// <summary>
/// ߼ڵĻ
/// </summary>
class COMPONENTBASIC_EXPORT LogicalNode : virtual public Serializable, public std::enable_shared_from_this<LogicalNode> {
private:
	int _node_id = 0;
	NodeKind _node_type = NodeKind::ACTIONNODE;

	std::weak_ptr<LogicalNode> _parent_bind;
	QList<std::shared_ptr<LogicalNode>> _child_list;

	/// <summary>
	/// øڵ
	/// </summary>
	/// <param name="pnode"></param>
	void _set_parent_node(std::weak_ptr<LogicalNode> pnode);

public:
	LogicalNode(NodeKind t);
	virtual ~LogicalNode() = default;

	/// <summary>
	/// ȡڵ
	/// </summary>
	/// <returns></returns>
	NodeKind nodeKind() const;
	/// <summary>
	/// ڵ
	/// </summary>
	/// <returns></returns>
	int depth() const;

	/// <summary>
	/// ȡͼڵ
	/// </summary>
	/// <returns></returns>
	std::shared_ptr<BehaviorMapNode> bindMap() const;

	/// <summary>
	/// ˽ڵ
	/// </summary>
	/// <returns></returns>
	std::weak_ptr<LogicalNode> parent() const;
	/// <summary>
	/// ȡӽڵб
	/// </summary>
	/// <returns></returns>
	virtual QList<std::shared_ptr<LogicalNode>> children() const;
	/// <summary>
	/// ӽڵ
	/// </summary>
	/// <param name="node"></param>
	/// <param name="index"></param>
	virtual void insert(std::shared_ptr<LogicalNode> node, int index = -1);
	/// <summary>
	/// Ƴӽڵ
	/// </summary>
	/// <param name="node"></param>
	virtual void remove(std::shared_ptr<LogicalNode> node);




	/// <summary>
	/// ýڵid
	/// </summary>
	/// <param name="unique_id"></param>
	void setID(int unique_id);
	/// <summary>
	/// ȡڵid
	/// </summary>
	/// <returns></returns>
	int getID() const;
	/// <summary>
	/// ʵڵ
	/// </summary>
	/// <returns></returns>
	QString rtName() const;

	/// <summary>
	/// ȡڵ
	/// </summary>
	/// <returns></returns>
	virtual QString typeName() const = 0;
	/// <summary>
	/// ڲʶݽӿ
	/// </summary>
	/// <returns>map{name, type-ins}</returns>
	virtual QHash<QString, std::shared_ptr<TopicData>> inputList() const = 0;
	/// <summary>
	/// ڲʶݽӿ
	/// </summary>
	/// <returns>map{name, type-ins}</returns>
	virtual QHash<QString, std::shared_ptr<TopicData>> outputList() const = 0;


	/// <summary>
	/// ڲ״̬
	/// </summary>
	virtual void reset() = 0;
	/// <summary>
	/// ȡִڽڵ
	/// </summary>
	/// <returns></returns>
	virtual QList<std::shared_ptr<LogicalNode>> getForwards() const = 0;
	/// <summary>
	/// нڵִ״̬ط
	/// </summary>
	/// <param name="node"></param>
	/// <param name="ste"></param>
	/// <returns>ִб־</returns>
	virtual bool fallback(std::shared_ptr<LogicalNode> node, LogicalResult ste) = 0;
	/// <summary>
	/// ִ߼ڵ
	/// </summary>
	/// <param name="core"></param>
	/// <returns></returns>
	virtual LogicalResult execute() = 0;


	/// <summary>
	/// ½Ĭʵ
	/// </summary>
	/// <returns></returns>
	virtual std::shared_ptr<Serializable> newDefault() const = 0;
};


/// <summary>
/// ߼ڵ
/// </summary>
enum class IO_TYPE {
	INPUT, OUTPUT
};
/// <summary>
/// ԶΪڵڵʵ
/// </summary>
class COMPONENTBASIC_EXPORT BehaviorMapNode : public LogicalNode {
private:
	std::shared_ptr<MapKernal> _bind_kernal;
	QString _map_name = u8"Ϊ";

	/// <summary>
	/// ͼ
	/// </summary>
	QHash<QString, std::pair<IO_TYPE, std::shared_ptr<TopicData>>> _variables;
	LogicalResult _state_value = LogicalResult::FAILURE;

public:
	BehaviorMapNode(std::shared_ptr<MapKernal> kernal);

	std::shared_ptr<MapKernal> getKernal() const;

	/// <summary>
	/// ñʹ׳UniException쳣
	/// ڵʼʱᰴעĬֵ
	/// </summary>
	virtual void setVariable(const QString& key, IO_TYPE t, std::shared_ptr<TopicData> ins);
	/// <summary>
	/// ȡָƵݱ
	/// </summary>
	/// <param name="key"></param>
	/// <returns>ָݣnullptrͲȷ</returns>
	virtual std::shared_ptr<TopicData> getVariable(const QString& key) const;


	/// <summary>
	/// Ϊڵ
	/// </summary>
	/// <param name="val"></param>
	virtual void resetName(const QString& val);
	/// <summary>
	/// ִ״̬
	/// </summary>
	void reset() override;
	/// <summary>
	/// ȡڵ
	/// </summary>
	/// <returns></returns>
	QString typeName() const override;

	/// <summary>
	/// ȡǰýڵ
	/// </summary>
	/// <returns></returns>
	QList<std::shared_ptr<LogicalNode>> getForwards() const override;
	/// <summary>
	/// ״̬
	/// </summary>
	/// <param name="node"></param>
	/// <param name="ste"></param>
	/// <returns></returns>
	bool fallback(std::shared_ptr<LogicalNode> node, LogicalResult ste) override;

	/// <summary>
	/// ִ״̬
	/// </summary>
	/// <returns></returns>
	LogicalResult execute() override;

	/// <summary>
	/// б
	/// </summary>
	/// <returns></returns>
	QHash<QString, std::shared_ptr<TopicData>> inputList() const override;
	/// <summary>
	/// б
	/// </summary>
	/// <returns></returns>
	QHash<QString, std::shared_ptr<TopicData>> outputList() const override;

	/// <summary>
	/// ʵ
	/// </summary>
	/// <param name="p"></param>
	/// <returns></returns>
	virtual std::shared_ptr<Serializable> newDefault() const override;
	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
};

// Ϊڵ ======================================================

/// <summary>
/// ˳ڵ
/// </summary>
class COMPONENTBASIC_EXPORT SequenceNode : public LogicalNode {
private:
	QHash<int, LogicalResult> _state_map;

public:
	SequenceNode();

	QList<std::shared_ptr<LogicalNode>> getForwards() const override;
	QString typeName() const override;

	QHash<QString, std::shared_ptr<TopicData>> inputList() const override;
	QHash<QString, std::shared_ptr<TopicData>> outputList() const override;

	void reset() override;
	LogicalResult execute() override;
	bool fallback(std::shared_ptr<LogicalNode> node, LogicalResult ste) override;

	virtual std::shared_ptr<Serializable> newDefault() const override;
	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
};

/// <summary>
/// ѡڵ
/// </summary>
class COMPONENTBASIC_EXPORT SelectorNode : public LogicalNode {
private:
	QHash<int, LogicalResult> _state_map;

public:
	SelectorNode();

	QList<std::shared_ptr<LogicalNode>> getForwards() const override;
	QString typeName() const override;

	QHash<QString, std::shared_ptr<TopicData>> inputList() const override;
	QHash<QString, std::shared_ptr<TopicData>> outputList() const override;

	void reset() override;
	LogicalResult execute() override;
	bool fallback(std::shared_ptr<LogicalNode> node, LogicalResult ste) override;

	virtual std::shared_ptr<Serializable> newDefault() const override;
	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
};

/// <summary>
/// нڵ
/// </summary>
class COMPONENTBASIC_EXPORT ParallelNode : public LogicalNode {
private:
	QHash<int, LogicalResult> _state_map;

public:
	ParallelNode();

	QList<std::shared_ptr<LogicalNode>> getForwards() const override;
	QString typeName() const override;

	QHash<QString, std::shared_ptr<TopicData>> inputList() const override;
	QHash<QString, std::shared_ptr<TopicData>> outputList() const override;

	void reset() override;
	LogicalResult execute() override;
	bool fallback(std::shared_ptr<LogicalNode> node, LogicalResult ste) override;

	virtual std::shared_ptr<Serializable> newDefault() const override;
	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
};

/// <summary>
/// ȽϽڵ
/// </summary>
class COMPONENTBASIC_EXPORT CompareNode : public LogicalNode {
private:
	std::shared_ptr<MapKernal> _bind_kernal;

	std::shared_ptr<CompareDelegate> _bind_delegate;
	QHash<QString, std::shared_ptr<TopicData>> _data_map;

public:
	CompareNode(std::shared_ptr<MapKernal> kernal);

	QString delegateName() const;
	void bindDelegate(std::shared_ptr<CompareDelegate> ins);

	virtual QList<std::shared_ptr<LogicalNode>> getForwards() const override;
	QString typeName() const override;

	QHash<QString, std::shared_ptr<TopicData>> inputList() const override;
	QHash<QString, std::shared_ptr<TopicData>> outputList() const override;

	virtual void insert(std::shared_ptr<LogicalNode> node, int index /*= -1*/) override;

	void reset() override;
	LogicalResult execute() override;
	virtual bool fallback(std::shared_ptr<LogicalNode> node, LogicalResult ste) override;

	virtual std::shared_ptr<Serializable> newDefault() const override;
	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
};

/// <summary>
/// ִнڵ
/// </summary>
class COMPONENTBASIC_EXPORT ExecuteNode : public LogicalNode {
private:
	std::shared_ptr<MapKernal> _bind_kernal;
	std::shared_ptr<ExecuteDelegate> _bind_delegate;

public:
	ExecuteNode(std::shared_ptr<MapKernal> kernal);

	QString delegateName() const;
	void bindDelegate(std::shared_ptr<ExecuteDelegate> ins);

	virtual QList<std::shared_ptr<LogicalNode>> getForwards() const override;
	QString typeName() const override;

	QHash<QString, std::shared_ptr<TopicData>> inputList() const override;
	QHash<QString, std::shared_ptr<TopicData>> outputList() const override;

	virtual void insert(std::shared_ptr<LogicalNode> node, int index /*= -1*/) override;

	void reset() override;
	LogicalResult execute() override;
	virtual bool fallback(std::shared_ptr<LogicalNode> node, LogicalResult ste) override;

	virtual std::shared_ptr<Serializable> newDefault() const override;
	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
};

// Ϊڵ ====================================================================

/// <summary>
/// Ϊ
/// </summary>
class COMPONENTBASIC_EXPORT BehaviorPerformPlugin :
	public RespondRoute<
	WsRespond<EntityInitialRequest, RespondDefault>,
	WsRespond<EntityPreparedRequest, RespondDefault>,
	WsRespond<SyncRequest, RespondDefault>,
	WsRespond<DeduceRequest, RespondDefault>>,
	public RequestRoute<>
{
private:
	std::weak_ptr<WsEntity> _bind_entity;

public:
	void mapReset(const QString& path);

	// ͨ ProcList ̳
	void execute(std::shared_ptr<const EntityInitialRequest> in, QList<std::shared_ptr<RespondDefault>>& out) override;
	void execute(std::shared_ptr<const EntityPreparedRequest> in, QList<std::shared_ptr<RespondDefault>>& out) override;
	void execute(std::shared_ptr<const SyncRequest> in, QList<std::shared_ptr<RespondDefault>>& out) override;
	void execute(std::shared_ptr<const DeduceRequest> in, QList<std::shared_ptr<RespondDefault>>& out) override;

	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;

	std::shared_ptr<Serializable> newDefault() const override;
	void bindEntity(std::weak_ptr<WsEntity> host) override;
	QString name() const override;
};

