#include "BehaviorEditor.h"

NodePresent::NodePresent(BehaviorsPresent* pwidget, double& width_bind, std::shared_ptr<LogicalNode> bind)
	: _widget_p(pwidget), _node_bind(bind), _column_width(width_bind) {
	this->setAcceptDrops(true);
}

QRectF NodePresent::contentMeasure() const
{
	auto metrics = this->_widget_p->fontMetrics();
	return metrics.boundingRect(this->_node_bind->rtName());
}

#include <QDebug>
AcceptType NodePresent::testAccept(const QPointF& local_pos, const QString& kind_str) const
{
	auto outline = this->boundingRect();
	auto width = outline.width();
	auto origin = outline.topLeft();

	auto left_rect = QRectF(origin + QPointF(0, 0), QSizeF(width / 3, outline.height()));
	auto right_rect = QRectF(origin + QPointF(width * 2 / 3, 0), QSizeF(width / 3, outline.height()));
	auto top_rect = QRectF(origin + QPointF(width / 3, outline.height() / 2), QSizeF(width / 3, outline.height() / 2));
	auto bottom_rect = QRectF(origin + QPointF(width / 3, 0), QSizeF(width / 3, outline.height() / 2)); 

	QList<NodeKind> no_child{ NodeKind::ACTIONNODE, NodeKind::COMPARENODE };
	auto pnode = this->_node_bind->parent().lock();
	decltype(pnode) new_type = nullptr;
	if (this->_node_bind->bindMap())
		new_type = this->_node_bind->bindMap()->getKernal()->getNode(kind_str);
	else
		new_type = std::dynamic_pointer_cast<BehaviorMapNode>(this->_node_bind)->getKernal()->getNode(kind_str);

	// ǰڵڸڵҲActionCompare
	if (left_rect.contains(local_pos) && pnode && !no_child.contains(new_type->nodeKind()))
		return AcceptType::PREV_LEVEL;

	// ǰڵڸڵҸڵԴڶڵ
	if (pnode && pnode->nodeKind() != NodeKind::MAPNODE) {
		if (top_rect.contains(local_pos))
			return AcceptType::PREV_SIBLING;

		if (bottom_rect.contains(local_pos))
			return AcceptType::NEXT_SIBLING;
	}

	// ֱ浱ǰڵǷԲ½ڵ
	if (right_rect.contains(local_pos))
		switch (this->_node_bind->nodeKind()) {
		case NodeKind::MAPNODE: // ڵֻһڵ
			if (!this->_node_bind->children().size() && !this->_node_bind->parent().lock())
				return AcceptType::NEXT_LEVEL;
			break;
		case NodeKind::PARALLELNODE:
		case NodeKind::SELECTORNODE:
		case NodeKind::SEQUENCENODE:
			return AcceptType::NEXT_LEVEL;
		default:	// ActionCompareڵ㲻ɽڵ
			break;
		}


	return AcceptType::NONE;
}

#include <QGraphicsSceneDragDropEvent>
#include <QMimeData>
void NodePresent::dragEnterEvent(QGraphicsSceneDragDropEvent* e)
{
	QGraphicsItem::dragEnterEvent(e);
	if (e->mimeData()->hasFormat("application/drag-node")) {
		e->acceptProposedAction();
	}
}

void NodePresent::dragLeaveEvent(QGraphicsSceneDragDropEvent* event)
{
	QGraphicsItem::dragLeaveEvent(event);
	this->_drop_target = AcceptType::NONE;
	this->update();
}

#include <QApplication>
void NodePresent::dragMoveEvent(QGraphicsSceneDragDropEvent* e)
{
	QGraphicsItem::dragMoveEvent(e);
	this->update();
	if (e->mimeData()->hasFormat("application/drag-node")) {
		auto kind_string = QString::fromUtf8(e->mimeData()->data("application/drag-node"));
		this->_drop_target = this->testAccept(e->pos(), kind_string);
		if (_drop_target != AcceptType::NONE) {
			e->acceptProposedAction();
			return;
		}
	}

	e->ignore();
}

void NodePresent::dropEvent(QGraphicsSceneDragDropEvent* e)
{
	auto kind_string = QString::fromUtf8(e->mimeData()->data("application/drag-node"));
	auto parent_node = this->_node_bind->parent().lock();
	decltype(parent_node) new_type = nullptr;
	if (this->_node_bind->bindMap())
		new_type = this->_node_bind->bindMap()->getKernal()->getNode(kind_string);
	else
		new_type = std::dynamic_pointer_cast<BehaviorMapNode>(this->_node_bind)->getKernal()->getNode(kind_string);
	auto new_node = std::dynamic_pointer_cast<LogicalNode>(new_type->newDefault());
	new_node->setID(++_widget_p->_node_id_max);

	auto this_node = this->_node_bind;
	auto appoint_index = 0;
	if(parent_node)
		appoint_index = parent_node->children().indexOf(this_node);

	switch (_drop_target) {
	case AcceptType::NONE:
		break;
	case AcceptType::PREV_LEVEL: {
		parent_node->remove(this_node);
		parent_node->insert(new_node, appoint_index);
		new_node->insert(this_node);
	}break;
	case AcceptType::NEXT_LEVEL:
		this_node->insert(new_node);
		break;
	case AcceptType::PREV_SIBLING:
		parent_node->insert(new_node, appoint_index);
		break;
	case AcceptType::NEXT_SIBLING:
		parent_node->insert(new_node, appoint_index + 1);
		break;
	}

	_widget_p->relayout();
	this->_drop_target = AcceptType::NONE;
	this->update();
}

QRectF NodePresent::boundingRect() const
{
	auto rect = contentMeasure();
	rect.setWidth(std::max(_column_width, rect.width()));
	_column_width = rect.width();

	return rect + QMargins(0, 0, 30, 30);
}

#include <QStyleOptionGraphicsItem>
void NodePresent::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
{
	painter->save();

	painter->drawRect(boundingRect());
	painter->fillRect(option->rect - QMargins(8, 8, 8, 8), Qt::gray);

	auto outline = this->boundingRect();
	auto width = outline.width();
	auto origin = outline.topLeft();

	painter->save();

	painter->translate(QPointF(15, 15));
	auto rect = contentMeasure();
	painter->drawText(rect, 0, this->_node_bind->rtName());

	painter->restore();


	QRectF paint_target_rect;
	switch (_drop_target) {
	case AcceptType::NONE:
		break;
	case AcceptType::PREV_LEVEL:
		paint_target_rect = QRectF(origin + QPointF(0, 0), QSizeF(width / 3, outline.height()));;
		break;
	case AcceptType::NEXT_LEVEL:
		paint_target_rect = QRectF(origin + QPointF(width * 2 / 3, 0), QSizeF(width / 3, outline.height()));;
		break;
	case AcceptType::PREV_SIBLING:
		paint_target_rect = QRectF(origin + QPointF(width / 3, outline.height() / 2), QSizeF(width / 3, outline.height() / 2));;
		break;
	case AcceptType::NEXT_SIBLING:
		paint_target_rect = QRectF(origin + QPointF(width / 3, 0), QSizeF(width / 3, outline.height() / 2));;
		break;
	}

	painter->fillRect(paint_target_rect, Qt::green);
	painter->restore();
}

BehaviorsPresent::BehaviorsPresent(QWidget* parent /*= nullptr*/)
	: QGraphicsView(parent) {
	this->setScene(&_bind_scene);

	this->setAcceptDrops(true);

	auto font = this->font();
	font.setPixelSize(20);
	this->setFont(font);
}

void BehaviorsPresent::setRoot(std::shared_ptr<BehaviorMapNode> root)
{
	qDeleteAll(_present_peers);
	_present_peers.clear();

	this->_bind_maproot = root;
	relayout();
}

void BehaviorsPresent::presentAllocate(std::shared_ptr<LogicalNode> ins)
{
	switch (ins->nodeKind())
	{
	case NodeKind::MAPNODE:
	case NodeKind::PARALLELNODE:
	case NodeKind::SEQUENCENODE:
	case NodeKind::SELECTORNODE:
		if (ins->nodeKind() != NodeKind::MAPNODE || !ins->bindMap()) {
			auto type = ins->nodeKind();
			for (auto it : ins->children())
				presentAllocate(it);
		}

	default:
		if (_present_peers.contains(ins))
			return;

		for (auto idx = _column_aligns.size(); idx < ins->depth() + 1; ++idx)
			_column_aligns.append(0);

		auto& current_width = _column_aligns[ins->depth()];
		_present_peers[ins] = new NodePresent(this, current_width, ins);
		this->_bind_scene.addItem(_present_peers[ins]);
		break;
	}
}

void BehaviorsPresent::presentRelease(std::shared_ptr<LogicalNode> ins)
{
	switch (ins->nodeKind())
	{
	case NodeKind::MAPNODE:
	case NodeKind::PARALLELNODE:
	case NodeKind::SELECTORNODE:
	case NodeKind::SEQUENCENODE: {
		for (auto item : ins->children()) {
			presentRelease(item);
		}
	}break;
	default:
		if (this->_present_peers.contains(ins))
			delete this->_present_peers[ins];
		this->_present_peers.remove(ins);
		break;
	}
}

void BehaviorsPresent::relayout()
{
	if (!_bind_maproot)
		return;

	// ʼʾڵ
	presentAllocate(_bind_maproot);

	QHash<std::shared_ptr<LogicalNode>, std::pair<QSizeF, QSizeF>> results;
	// ߴ
	outlineMeasure(_bind_maproot, results);
	// ԪŲ
	nodeRelayout(results, _bind_maproot, QPointF());
}

QSizeF BehaviorsPresent::outlineMeasure(std::shared_ptr<LogicalNode> ins, QHash<std::shared_ptr<LogicalNode>, std::pair<QSizeF, QSizeF>>& _outline_occupy)
{
	QSizeF outline_box(0, 0);

	switch (ins->nodeKind()) {
	case NodeKind::MAPNODE:
	case NodeKind::PARALLELNODE:
	case NodeKind::SELECTORNODE:
	case NodeKind::SEQUENCENODE:
		// ӽڵ㼯ϳߴ磬ڵΪڵ
		if (ins->nodeKind() != NodeKind::MAPNODE || !ins->bindMap()) {
			for (auto nit : ins->children()) {
				auto nit_size = outlineMeasure(nit, _outline_occupy);
				outline_box.setWidth(std::max(outline_box.width(), nit_size.width() + _space_h));
				outline_box += QSizeF(0, nit_size.height());
			}
		}
	default: {
		auto root_peer = _present_peers[ins];
		auto root_box = root_peer->boundingRect();
		outline_box += QSizeF(root_box.width(), 0);
		outline_box.setHeight(std::max(outline_box.height(), root_box.height()));

		_outline_occupy[ins] = std::make_pair(outline_box, root_box.size());
		return outline_box;
	}break;
	}
}

QPointF BehaviorsPresent::nodeRelayout(QHash<std::shared_ptr<LogicalNode>, std::pair<QSizeF, QSizeF>>& _outline_occupy,
	std::shared_ptr<LogicalNode> ins, const QPointF& origin_offset) {
	// ȡߴ¼
	auto outline_occupy = _outline_occupy[ins];
	auto node_outline = outline_occupy.first;
	auto node_occupy = outline_occupy.second;

	// ȡʾڵ
	auto present_node = _present_peers[ins];

	// ýڵλ
	QPointF local_origin_offset(0, (node_outline.height() - node_occupy.height()) / 2);
	present_node->setPos(origin_offset + local_origin_offset);

	// ӽڵλ
	QPointF child_origin = origin_offset + QPointF(node_occupy.width() + _space_h, 0);
	switch (ins->nodeKind())
	{
	case NodeKind::ACTIONNODE:
	case NodeKind::COMPARENODE:
		break;
	default:
		// ӽڵ㼯λãڵǶ˸ڵ
		if (ins->nodeKind() != NodeKind::MAPNODE || !ins->bindMap()) {
			auto child_list = ins->children();
			for (auto iter = child_list.rbegin(); iter != child_list.rend(); ++iter) {
				auto item = *iter;
				child_origin = nodeRelayout(_outline_occupy, item, child_origin);
			}
		}break;
	}

	// ½λ
	return origin_offset + QPointF(0, node_outline.height());
}

uint qHash(const std::shared_ptr<LogicalNode> data, uint seed) noexcept
{
	return qHash((void*)data.get(), seed);
}

#include <QSplitter>
#include <QMenuBar>
#include <QMenu>
#include <QFileDialog>
#include <QDebug>

void BehaviorEditor::open_behavior_map()
{
	auto url = QFileDialog::getOpenFileUrl(this, u8"Ϊļ", QUrl(), "*.behw");

	this->_logical_present->setRoot(_map_root);

	if (!url.isValid())
		return;
	qDebug() << url;
}

BehaviorEditor::BehaviorEditor(QWidget* parent /*= nullptr*/)
	: QMainWindow(parent),
	_type_view(new NodeTypesView(this)),
	_type_model(new QStandardItemModel(this)),
	_logical_present(new BehaviorsPresent(this)),
	_logs_present(new QTextBrowser(this))
{
	_global_loader = std::make_shared<MessageLoader>();
	_global_kernal = std::make_shared<MapKernal>(_global_loader);
	_global_kernal->initial();

	this->_map_root = std::make_shared<BehaviorMapNode>(_global_kernal);

	auto mbar = this->menuBar();
	auto _file = mbar->addMenu(u8"ļ");
	_file->addAction(u8"", this, &BehaviorEditor::open_behavior_map);

	auto font = this->font();
	font.setPixelSize(20);
	_type_view->setFont(font);

	auto split_v = new QSplitter(Qt::Vertical, this);
	this->setCentralWidget(split_v);

	auto split_h = new QSplitter(Qt::Horizontal, this);
	split_v->addWidget(split_h);
	split_v->addWidget(_logs_present);

	split_h->addWidget(_logical_present);
	split_h->addWidget(_type_view);
	_type_view->setModel(_type_model);

	nodeTypesViewInit(_type_model);
}

#include <QStandardItem>
void BehaviorEditor::nodeTypesViewInit(QStandardItemModel* m)
{
	auto types = this->_global_kernal->nodeTypes();
	std::sort(types.begin(), types.end());
	for (auto type : types) {
		auto row_item = new QStandardItem(type);
		row_item->setEditable(false);
		m->appendRow(row_item);
	}
}

NodeTypesView::NodeTypesView(QWidget* parent /*= nullptr*/)
	: QListView(parent)
{
	this->setDragEnabled(true);
	this->setDragDropMode(QAbstractItemView::DragOnly);
	this->setSelectionBehavior(QAbstractItemView::SelectRows);
	this->setSelectionMode(QAbstractItemView::SingleSelection);
}

#include <QVariant>
#include <QDrag>
#include <QPainter>
void NodeTypesView::startDrag(Qt::DropActions supported)
{
	auto selected = this->selectedIndexes();
	if (selected.size() && selected.first().isValid()) {
		auto val = this->model()->data(selected.first(), Qt::DisplayRole);
		auto type_name = val.toString();

		// ԶMIME
		auto mime_data = new QMimeData();
		mime_data->setData("application/drag-node", type_name.toUtf8());

		auto view_rect = this->visualRect(selected.first());
		QPixmap pixmap(view_rect.size());
		pixmap.fill(Qt::transparent);
		QPainter p(&pixmap);
		this->render(&p, QPoint(), QRegion(view_rect));
		p.end();

		// ϶
		auto drag = new QDrag(this);
		drag->setMimeData(mime_data);
		drag->setPixmap(pixmap);

		auto result = drag->exec(supported);
		if (result == Qt::CopyAction) {
			qDebug() << __FILE__ << __LINE__;
		}
	}
}
