#pragma once
#include <QMainWindow>
#include "BehaviorPerform.h"
#include <QGraphicsItem>
#include <QGraphicsView>

uint qHash(const std::shared_ptr<LogicalNode> data, uint seed) noexcept;

enum class AcceptType {
	NONE = 0, PREV_LEVEL, NEXT_LEVEL, PREV_SIBLING, NEXT_SIBLING
};

class BehaviorsPresent;
/// <summary>
/// ڵʾ
/// </summary>
class NodePresent : public QGraphicsItem {
private:
	BehaviorsPresent* const _widget_p;
	std::shared_ptr<LogicalNode> _node_bind;
	double& _column_width;

	AcceptType _drop_target = AcceptType::NONE;

public:
	NodePresent(BehaviorsPresent* pwidget, double& width_bind, std::shared_ptr<LogicalNode> bind);

	QRectF contentMeasure() const;
	AcceptType testAccept(const QPointF& local_pos, const QString &kind_str) const;

	// ͨ QGraphicsItem ̳
	QRectF boundingRect() const override;
	void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget) override;

protected:
	virtual void dragEnterEvent(QGraphicsSceneDragDropEvent* event);
	virtual void dragLeaveEvent(QGraphicsSceneDragDropEvent* event);
	virtual void dragMoveEvent(QGraphicsSceneDragDropEvent* event);
	virtual void dropEvent(QGraphicsSceneDragDropEvent* event);
};


/// <summary>
/// Ϊ༭չʾ
/// </summary>
class BehaviorsPresent : public QGraphicsView
{
private:
	QGraphicsScene _bind_scene;

	const double _space_h = 120;
	QVector<double> _column_aligns;
	QHash<std::shared_ptr<LogicalNode>, NodePresent*> _present_peers;

	/// <summary>
	/// Ϊڵ
	/// </summary>
	std::shared_ptr<BehaviorMapNode> _bind_maproot = nullptr;

public:
	uint64_t _node_id_max = 0;
	BehaviorsPresent(QWidget* parent = nullptr);

	/// <summary>
	/// 滻Ϊ
	/// </summary>
	/// <param name="root"></param>
	void setRoot(std::shared_ptr<BehaviorMapNode> root);

	/// <summary>
	/// ʾڵ
	/// </summary>
	/// <param name="_present_peers"></param>
	/// <param name="ins"></param>
	void presentAllocate(std::shared_ptr<LogicalNode> ins);
	/// <summary>
	/// ݹͷʾڵ
	/// </summary>
	/// <param name="ins"></param>
	void presentRelease(std::shared_ptr<LogicalNode> ins);

	/// <summary>
	/// ²
	/// </summary>
	void relayout();
	/// <summary>
	/// ߴ
	/// </summary>
	/// <param name="ins"></param>
	/// <param name="_outline_occupy"></param>
	/// <returns></returns>
	QSizeF outlineMeasure(std::shared_ptr<LogicalNode> ins, QHash<std::shared_ptr<LogicalNode>, std::pair<QSizeF, QSizeF>>& _outline_occupy);
	/// <summary>
	/// ڵ㲼
	/// </summary>
	/// <param name="_outline_occupy">ߴ¼</param>
	/// <param name="ins">ָڵ</param>
	/// <param name="origin_offset">Ͻ</param>
	/// <returns>½</returns>
	QPointF nodeRelayout(QHash<std::shared_ptr<LogicalNode>, std::pair<QSizeF, QSizeF>>& _outline_occupy,
		std::shared_ptr<LogicalNode> ins, const QPointF& origin_offset);
};


#include <QListView>
class NodeTypesView : public QListView {
public:
	NodeTypesView(QWidget* parent = nullptr);

protected:
	virtual void startDrag(Qt::DropActions supported) override;
};


#include <QStandardItemModel>
#include <QTextBrowser>
#include <MessageLoader.h>
class BehaviorEditor : public QMainWindow {
private:
	std::shared_ptr<MessageLoader> _global_loader;
	std::shared_ptr<MapKernal> _global_kernal;

	NodeTypesView* const _type_view;
	QStandardItemModel* const _type_model;
	BehaviorsPresent* const _logical_present;
	QTextBrowser* const _logs_present;

	std::shared_ptr<BehaviorMapNode> _map_root;

	void open_behavior_map();

public:
	BehaviorEditor(QWidget* parent = nullptr);

	void nodeTypesViewInit(QStandardItemModel* m);
};